require "import"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"
import "java.util.Locale"

-- Define a variable to store the spoken words
local spokenText = ""

-- Define a table for emojis based on Hindi words
local emojiDictionary = {
    -- Common words with emojis
    खुश = "😊",
    दुखी = "😢",
    प्यार = "❤️",
    गुस्सा = "😡",
    कूल = "😎",
    अच्छा = "👍",
    बुरा = "👎",
    हंसी = "😂",
    धन्यवाद = "🙏",
    खुशी = "😄",
    मुस्कान = "😊",
    नमस्ते = "🙏",
    नमस्कार = "👋",
    क्या_हाल_है = "😃",
    कैसे_हो = "😄",
    शाबाश = "👏",
    ध्यान = "🧘‍♂️",
    चिंता = "😟",
    चलो = "🏃‍♂️",
    चाय = "☕",
    पानी = "💧",
    खाना = "🍽️",
    सोना = "💤",
    काम = "💼",
    स्कूल = "🏫",
    दोस्त = "👫",
    परिवार = "👨‍👩‍👧‍👦",
    धन्यवाद = "🙏",
    सस्ता = "💸",
    महंगा = "💰",
    यात्रा = "✈️",
    छुट्टी = "🏖️",
    हास्य = "😂",
    प्यार_भरा = "💖",
    शांति = "🕊️",
    मुस्कान = "😊",
    मित्रता = "🤝",
    लक्ष्य = "🎯",
    सेहत = "💪",
    खुशी_का_समय = "🎉",
    उत्सव = "🎊",
    कामयाबी = "🏆",
    सीखना = "📚"
}

-- Function to store recognized words
function saveWords(words)
    spokenText = words
end

-- Function to insert emoji after every word based on the emoji dictionary
function addEmojiAfterWords(text)
    local words = {}
    -- Split the text into words using whitespace as separator
    for word in text:gmatch("%S+") do
        table.insert(words, word)
    end
    
    local result = ""
    -- Loop through each word and add it along with its emoji
    for _, word in ipairs(words) do
        result = result .. word .. " "
        -- Check if the word exists in the emojiDictionary
        if emojiDictionary[word] then
            result = result .. emojiDictionary[word] .. " "
        end
    end
    return result
end

-- Function to start listening for speech input
function startListening()
    local verbalizationDetected = false

    -- Check if EditText node is available
    local node = service.getEditText()
    if node == nil then
        -- If no EditText node is present, send a message
        print("No text box is present.")
        return false
    end

    -- Create a new intent for speech recognition
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "hi-IN") -- Set the language to Hindi (hi-IN)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())

    -- Create a speech recognizer instance
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)

    -- Define a recognition listener for speech events
    local speechListener = RecognitionListener{
        onResults = function(results)
            local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data ~= nil and data.size() > 0 then
                verbalizationDetected = true
                local recognizedText = data.get(0)

                -- Save the recognized words
                saveWords(recognizedText)

                -- Get the EditText node
                local editText = service.getText(node)

                -- Add emoji after every word based on the dictionary
                local textWithEmojis = addEmojiAfterWords(recognizedText)

                -- Insert recognized words with emojis into the EditText node
                service.insertText(node, textWithEmojis)
            end

            -- If no verbalization detected, stop listening
            if not verbalizationDetected then
                kill()
                return false
            end
        end,
        onError = function()
            -- If an error occurs, stop listening
            kill()
            return false
        end
    }

    -- Set the recognition listener for the speech recognizer
    speechRecognizer.setRecognitionListener(speechListener)
    -- Start listening for speech input
    speechRecognizer.startListening(recognizerIntent)
end

-- Call the function to start listening for speech input
startListening()
-- Return true to indicate successful execution
return true